@@ -1,20 +1,20 @@
 /*******************************************************************************
    SecureSkat.cc, secure multiplayer implementation of german card game "Skat"
 
- Copyright (C) 2002, 2003, 2004 Heiko Stamer, <stamer@gaos.org>
+ Copyright (C) 2002, 2003, 2004, 2005  Heiko Stamer <stamer@gaos.org>
 
-   This program is free software; you can redistribute it and/or modify
+   SecureSkat is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful,
+   SecureSkat is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
+   along with SecureSkat; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *******************************************************************************/
 
@@ -23,7 +23,7 @@
 	#include "config.h"
 #endif
 
-// C/C++ header
+// C and C++ header
 #include <stdio.h>
 #include <cstdlib>
 #include <cstdarg>
@@ -61,6 +61,9 @@
 // libTMCG
 #include <libTMCG.hh>
 
+// libgnutls
+#include <gnutls/gnutls.h>
+
 #include "socketstream.hh"
 #include "securesocketstream.hh"
 #include "pipestream.hh"
@@ -481,27 +484,25 @@ void skat_connect
 	}
 	
 	// exchange secret keys for securesocketstreams
-	assert(gcry_md_test_algo(TMCG_GCRY_MD_ALGO) == 0);
-	char *key1 = new char[gcry_md_get_algo_dlen(TMCG_GCRY_MD_ALGO)];
-	char *key2 = new char[gcry_md_get_algo_dlen(TMCG_GCRY_MD_ALGO)];
-	gcry_randomize((unsigned char*)key1,
-		gcry_md_get_algo_dlen(TMCG_GCRY_MD_ALGO), GCRY_STRONG_RANDOM);
+	char *key1 = new char[TMCG_SAEP_S0], *key2 = new char[TMCG_SAEP_S0],
+		*dv = NULL;
+	gcry_randomize((unsigned char*)key1, TMCG_SAEP_S0, GCRY_STRONG_RANDOM);
 	*neighbor << pkr.key[pkr_idx].encrypt(key1) << std::endl << std::flush;
 	
 	neighbor->getline(tmp, sizeof(tmp));
-	const char *dv = sec.decrypt(tmp);
-	if (dv == NULL)
+	if (!sec.decrypt(dv, tmp))
 	{
-		std::cerr << _("TMCG: DecryptValue() failed") << std::endl;
+		std::cerr << _("TMCG: *.decrypt() failed") << std::endl;
 		delete neighbor;
+		delete [] key1, delete [] key2;
 		close(handle);
 		*out_pipe << "PART #openSkat_" << nr << std::endl << std::flush;
 		sleep(1),	exit(-6);
 	}
-	memcpy(key2, dv, gcry_md_get_algo_dlen(TMCG_GCRY_MD_ALGO));
+	memcpy(key2, dv, TMCG_SAEP_S0);
 	delete neighbor;
 	secure = new iosecuresocketstream(handle, key1, 16, key2, 16);
-	delete [] key1, delete [] key2;
+	delete [] key1, delete [] key2, delete [] dv;
 }
 
 void skat_accept (opipestream *out_pipe, int ipipe, const std::string &nr, int r,
@@ -575,27 +576,27 @@ void skat_accept (opipestream *out_pipe, int ipipe, const std::string &nr, int r
 							*neighbor << sec.sign(st) << std::endl << std::flush;
 							
 							// exchange secret keys for securesocketstreams
-							assert(gcry_md_test_algo(TMCG_GCRY_MD_ALGO) == 0);
-							char *key1 = new char[gcry_md_get_algo_dlen(TMCG_GCRY_MD_ALGO)];
-							char *key2 = new char[gcry_md_get_algo_dlen(TMCG_GCRY_MD_ALGO)];
+							char *key1 = new char[TMCG_SAEP_S0],
+								*key2 = new char[TMCG_SAEP_S0], *dv = NULL;
 							neighbor->getline(tmp, sizeof(tmp));
-							const char *dv = sec.decrypt(tmp);
-							if (dv == NULL)
+							if (!sec.decrypt(dv, tmp))
 							{
-								std::cerr << _("TMCG: DecryptValue() failed") << std::endl;
+								std::cerr << _("TMCG: *.decrypt() failed") << std::endl;
 								delete neighbor;
+								delete [] key1, delete [] key2;
 								close(handle);
 								*out_pipe << "PART #openSkat_" << nr << std::endl << std::flush;
 								sleep(1),	exit(-6);
 							}
-							memcpy(key2, dv, gcry_md_get_algo_dlen(TMCG_GCRY_MD_ALGO));
+							memcpy(key2, dv, TMCG_SAEP_S0);
 							
-							gcry_randomize((unsigned char*)key1,
-								gcry_md_get_algo_dlen(TMCG_GCRY_MD_ALGO), GCRY_STRONG_RANDOM);
-							*neighbor << pkr.key[pkr_idx].encrypt(key1) << std::endl << std::flush;
+							gcry_randomize((unsigned char*)key1, TMCG_SAEP_S0,
+								GCRY_STRONG_RANDOM);
+							*neighbor << pkr.key[pkr_idx].encrypt(key1) << std::endl <<
+								std::flush;
 							delete neighbor;
 							secure = new iosecuresocketstream(handle, key1, 16, key2, 16);
-							delete [] key1, delete [] key2;
+							delete [] key1, delete [] key2, delete [] dv;
 							break;
 						}
 						else
@@ -998,6 +999,12 @@ int skat_child
 	
 	// start game
 	std::cout << X << _("Table") << " " << nr << " " << _("start the game.") << std::endl;
+	if (neu)
+	{
+		// set topic
+		*out_pipe << "TOPIC #openSkat_" << nr << " :" << PACKAGE_STRING	<<
+			std::endl << std::flush;
+	}
 	int exit_code = skat_game(nr, r, pkr_self, neu, opipe, ipipe, ctl_o, ctl_i,
 		gp_tmcg, pkr, sec, right_neighbor, left_neighbor, vnicks, hpipe, pctl,
 		ipipe_readbuf, ipipe_readed);
@@ -1054,7 +1061,7 @@ void read_after_select(fd_set rfds, std::map<pid_t, int> &read_pipe, int what)
 				for (int i = 0; i < readed[pi->second]; i++)
 					if (readbuf[pi->second][i] == '\n')
 						cnt_delim++, pos_delim.push_back(i);
-				if (what == 1)
+				if (what == 1) // mutex update of ranking data from RNK childs
 				{
 					while (cnt_delim >= 2)
 					{
@@ -1073,7 +1080,7 @@ void read_after_select(fd_set rfds, std::map<pid_t, int> &read_pipe, int what)
 						rnk[rnk1] = rnk2;
 					}
 				}
-				else if (what == 2)
+				else if (what == 2) // mutex IRC output from game childs
 				{
 					while (cnt_delim >= 1)
 					{
@@ -1083,7 +1090,9 @@ void read_after_select(fd_set rfds, std::map<pid_t, int> &read_pipe, int what)
 							pos_delim[pos] - cnt_pos);
 						--cnt_delim, cnt_pos = pos_delim[pos] + 1, pos++;
 						std::string irc1 = tmp;
-						
+
+//std::cerr << "to IRC: " << irc1 << std::endl;
+
 						// do operation
 						if (strncasecmp(irc_command(irc1), "PRIVMSG", 7) == 0)
 						{
@@ -1095,8 +1104,8 @@ void read_after_select(fd_set rfds, std::map<pid_t, int> &read_pipe, int what)
 									// sign message
 									*irc << "PRIVMSG " << irc_parvec[0] <<
 										" :" << irc_parvec[1] << "~~~" <<
-										sec.sign(irc_parvec[1]) << std::endl
-										<< std::flush;
+										sec.sign(irc_parvec[1]) << std::endl <<
+										std::flush;
 								}
 								else if (irc_parvec[0] == "#openSkat")
 								{
@@ -1161,7 +1170,7 @@ void read_after_select(fd_set rfds, std::map<pid_t, int> &read_pipe, int what)
 							*irc << irc1 << std::endl << std::flush;
 					}
 				}
-				else if (what == 3)
+				else if (what == 3) // mutex import from PKI childs
 				{
 					while (cnt_delim >= 2)
 					{
@@ -1294,7 +1303,7 @@ static void process_line(char *line)
 				}
 			}
 		}
-		else if ((cmd_argv[0] == "rooms") || (cmd_argv[0] == "r?me"))
+		else if ((cmd_argv[0] == "rooms") || (cmd_argv[0] == "raeume"))
 		{
 			for (std::list<std::string>::const_iterator ti = tables.begin(); 
 				ti != tables.end(); ti++)
@@ -1763,26 +1772,26 @@ static void process_line(char *line)
 			}
 			else
 				std::cout << X << _("wrong number of arguments") << ": " << cmd_argc << 
-					std::endl << X << _("/help shows the std::list of commands") << std::endl;
+					std::endl << X << _("/help shows the list of commands") << std::endl;
 		}
 		else if ((cmd_argv[0] == "help") || (cmd_argv[0] == "hilfe"))
 		{
 			std::cout << XX << _("/quit") << " -- " <<
 				_("quit SecureSkat") << std::endl;
 			std::cout << XX << _("/on") << " -- " <<
-				_("turn output of IRC channel #openSkat on") << std::endl;
+				_("turn the output of IRC channel #openSkat on") << std::endl;
 			std::cout << XX << ("/off") << " -- " <<
-				_("turn output of IRC channel #openSkat off") << std::endl;
+				_("turn the output of IRC channel #openSkat off") << std::endl;
 			std::cout << XX << ("/players") << " -- " <<
-				_("show std::list of possible participants") << std::endl;
+				_("show the list of possible participants") << std::endl;
 			std::cout << XX << ("/tables") << " -- " <<
-				_("show std::list of existing game tables") << std::endl;
+				_("show the list of existing game tables") << std::endl;
 			std::cout << XX << ("/rooms") << " -- " <<
-				_("show std::list of existing voting rooms") << std::endl;
+				_("show the list of existing voting rooms") << std::endl;
 			std::cout << XX << _("/rank") << " -- " <<
-				_("show your current rank in all score std::lists") << std::endl;
+				_("show your current rank in all score lists") << std::endl;
 			std::cout << XX << _("/ballot") << " <nr> <bits> -- " <<
-				_("create room <nr> for voting between 2^<bits> values") << std::endl;
+				_("create the room <nr> for voting between 2^<bits> values") << std::endl;
 			std::cout << XX << _("/ballot") << " <nr> -- " <<
 				_("join the voting in room <nr>") << std::endl;
 			std::cout << XXX << "/<nr> open -- " <<
@@ -1790,28 +1799,28 @@ static void process_line(char *line)
 			std::cout << XXX << "/<nr> vote <r> -- " <<
 				_("vote in room <nr> for value <r>") << std::endl;
 			std::cout << XX << "/skat <nr> <r> -- " <<
-				_("create table <nr> for playing <r> rounds") << std::endl;
+				_("create the table <nr> for playing <r> rounds") << std::endl;
 			std::cout << XX << "/skat <nr> -- " <<
 				_("join the game on table <nr>") << std::endl;
 			std::cout << XX << "/<nr> <cmd> -- " <<
-				_("execute command <cmd> on table <nr>") << ":" << std::endl;
+				_("execute the command <cmd> on table <nr>") << ":" << std::endl;
 			std::cout << XXX << "/<nr> blatt --- " <<
-				_("show own cards and additional information") << std::endl;
+				_("show your own cards and additional information") << std::endl;
 			std::cout << XXX << "/<nr> reize --- " <<
 				_("bid or justify a bid") << std::endl;
 			std::cout << XXX << "/<nr> passe --- " <<
-				_("pass biding") << std::endl;
+				_("pass the biding") << std::endl;
 			std::cout << XXX << "/<nr> hand --- " <<
 				_("play without taking the two cards") << std::endl;
 			std::cout << XXX << "/<nr> skat --- " <<
-				_("take the two cards and show all") << std::endl;
+				_("take the two cards and show them") << std::endl;
 			std::cout << XXX << "/<nr> druecke <k1> <k2> --- " <<
-				_("put away card <k1> and <k2>") << std::endl;
+				_("put away the cards <k1> and <k2>") << std::endl;
 			std::cout << XXX << "/<nr> sagean <spiel> [zusatz] --- " <<
-				_("announce game <spiel> ([zusatz] is optional)") << std::endl;
+				_("announce the game <spiel> ([zusatz] is optional)") << std::endl;
 			std::cout << XXX << "/<nr> lege <k1> --- " <<
-				_("play card <k1>") << std::endl;
-			std::cout << XX << "<nr> " << _("arbitrary std::string") << std::endl;
+				_("play the card <k1>") << std::endl;
+			std::cout << XX << "<nr> " << _("arbitrary string") << std::endl;
 			std::cout << XX << "<r> " << _("unsigned integer") << std::endl;
 			std::cout << XXX << "<k1>, <k2> ::= { Sc, Ro, Gr, Ei } || " <<
 				"{ 7, 8, 9, U, O, K, 10, A }" << std::endl;
@@ -1838,7 +1847,7 @@ static void process_line(char *line)
 			}
 			if (!found)
 				std::cout << X << _("unknown command") << ": \"/" << cmd_argv[0] << 
-					"\"" << std::endl << X << _("/help shows the std::list of commands") << std::endl;
+					"\"" << std::endl << X << _("/help shows the list of commands") << std::endl;
 		}
 	}
 	else
@@ -1877,7 +1886,6 @@ void run_irc()
 #endif
 		MFD_SET(irc_handle, &rfds);
 		MFD_SET(pki7771_handle, &rfds);
-		MFD_SET(pki7772_handle, &rfds);
 		MFD_SET(rnk7773_handle, &rfds);
 		MFD_SET(rnk7774_handle, &rfds);
 		
@@ -2064,7 +2072,8 @@ void run_irc()
 				
 				if (num == 0)
 				{
-					std::cerr << _("IRC ERROR: connection with server collapsed") << std::endl;
+					std::cerr << _("IRC ERROR: connection with server collapsed") <<
+						std::endl;
 					break;
 				}
 				
@@ -2098,7 +2107,7 @@ void run_irc()
 				}
 				
 				if (strncasecmp(irc_command(irc_reply), "PING", 4) == 0)
-				{				
+				{
 					*irc << "PONG " << irc_params(irc_reply) << std::endl << std::flush;
 				}
 				else if (strncasecmp(irc_command(irc_reply), "001", 3) == 0)
@@ -2143,8 +2152,8 @@ void run_irc()
 						{
 							std::string tb = 
 								irc_parvec[0].substr(10, irc_parvec[0].length() - 10);
-								
-							if (nick.find(pub.keyid(), 0)	== 0)
+							
+							if (nick.find(pub.keyid(), 0) == 0)
 							{
 								std::cout << X << _("you join") << " " << tb << std::endl;
 							}
@@ -2184,8 +2193,8 @@ void run_irc()
 					(strncasecmp(irc_command(irc_reply), "405", 3) == 0) ||
 					(strncasecmp(irc_command(irc_reply), "475", 3) == 0))
 				{
-//std::cerr << "IRC: could not join to channel, channel error:" << std::endl;
-//std::cerr << irc_reply << std::endl;
+					std::cerr << "IRC ERROR: could not join to channel" << std::endl;
+					std::cerr << irc_reply << std::endl;
 				}
 				else if (strncasecmp(irc_command(irc_reply), "PART", 4) == 0)
 				{
@@ -2193,7 +2202,7 @@ void run_irc()
 					{
 						if (irc_parvec[0] == "#openSkat")
 						{
-							if (nick.find(pub.keyid(), 0)	== 0)
+							if (nick.find(pub.keyid(), 0) == 0)
 							{
 								std::cout << X << _("you leave channel") << " " <<
 									irc_parvec[0] << std::endl;
@@ -2201,8 +2210,8 @@ void run_irc()
 							else if (nick.find(public_prefix, 0) == 0)
 							{
 								nick_players.erase(nick), nick_sl.erase(nick);
-								nick_p7771.erase(nick),	nick_p7772.erase(nick);
-								nick_p7773.erase(nick),	nick_p7774.erase(nick);
+								nick_p7771.erase(nick), nick_p7772.erase(nick);
+								nick_p7773.erase(nick), nick_p7774.erase(nick);
 								if (nick_key.find(nick) != nick_key.end())
 									nick = nick_key[nick].name;
 								if (irc_stat)
@@ -2291,12 +2300,12 @@ void run_irc()
 									std::cout << X << _("reason") << ": " << irc_parvec[2] << std::endl;
 							}
 						}
-						else if ((irc_parvec[0].find("#openSkat_", 0) == 0) && 
+						else if ((irc_parvec[0].find("#openSkat_", 0) == 0) &&
 							(irc_parvec[0].length() > 10))
 						{
 							std::string tb = 
 								irc_parvec[0].substr(10, irc_parvec[0].length() - 10);
-								
+							
 							if (irc_parvec[1].find(pub.keyid(), 0) == 0)
 							{
 								if (games_tnr2pid.find(tb) != games_tnr2pid.end())
@@ -2344,7 +2353,7 @@ void run_irc()
 					}
 					else if (nick.find(public_prefix, 0) == 0)
 					{
-						nick_players.erase(nick),nick_sl.erase(nick); 
+						nick_players.erase(nick),nick_sl.erase(nick);
 						nick_p7771.erase(nick), nick_p7772.erase(nick);
 						nick_p7773.erase(nick), nick_p7774.erase(nick);
 						for (std::map<std::string, pid_t>::const_iterator gi = games_tnr2pid.begin();
@@ -2425,8 +2434,8 @@ void run_irc()
 											" " << irc_parvec[1] << std::endl;
 								}
 							}
-							else if ((irc_parvec[1].find("#openSkat_", 0) == 0) && 
-							(irc_parvec[1].length() > 10))
+							else if ((irc_parvec[1].find("#openSkat_", 0) == 0) &&
+								(irc_parvec[1].length() > 10))
 							{
 								std::string tb = 
 									irc_parvec[1].substr(10, irc_parvec[1].length() - 10);
@@ -2468,8 +2477,8 @@ void run_irc()
 					if (irc_paramvec(irc_params(irc_reply)) >= 2)
 					{
 						// control messages
-						if ((irc_parvec[0].find("#openSkat_", 0) == 0) && 
-							(irc_parvec[0].length() > 10) && 
+						if ((irc_parvec[0].find("#openSkat_", 0) == 0) &&
+							(irc_parvec[0].length() > 10) &&
 							(nick.find(public_prefix, 0) == 0))
 						{
 							std::string tb = irc_parvec[0].substr(10, irc_parvec[0].length() - 10);
@@ -2492,7 +2501,7 @@ void run_irc()
 											delete npipe;
 										}
 										else
-											std::cerr << _("TMCG: VerifyData() failed") << std::endl;
+											std::cerr << _("TMCG: verify() failed") << std::endl;
 									}
 									else
 										std::cerr << _("TMCG: no public key available") << std::endl;
@@ -2516,7 +2525,7 @@ void run_irc()
 									std::cout << "<" << nick << "> " << realmsg << std::endl;
 								}
 								else
-									std::cerr << _("TMCG: VerifyData() failed") << std::endl;
+									std::cerr << _("TMCG: verify() failed") << std::endl;
 							}
 							else
 								std::cout << "<?" << nick << "?> " << irc_parvec[1] << std::endl;
@@ -2589,9 +2598,18 @@ void run_irc()
 						}
 					}
 				}
+				else if (strncasecmp(irc_command(irc_reply), "NOTICE", 6) == 0)
+				{
+					if (irc_paramvec(irc_params(irc_reply)) >= 2)
+					{
+						std::cout << "[NOTICE] " << irc_parvec[1] << std::endl;
+					}
+				}
 				else
 				{ 
 					// unparsed IRC-message -- ignore this one
+//std::cerr << "IRC: unparsed message:" << std::endl;
+//std::cerr << irc_reply << std::endl;
 				}
 				
 					}
@@ -3002,14 +3020,14 @@ void init_term()
 	{
 		perror("init_term (tcgetattr)");
 		exit(-1);
-	} 
+	}
 	new_term = old_term;
 	new_term.c_lflag &= ~ICANON, new_term.c_cc[VTIME] = 1;
 	if (tcsetattr(fileno(stdin), TCSANOW, &new_term) < 0)
 	{
 		perror("init_term (tcsetattr)");
 		exit(-1);
-	} 
+	}
 	rl_readline_name = "SecureSkat";
 #ifdef _RL_FUNCTION_TYPEDEF
 	rl_callback_handler_install(NULL, (rl_vcpfunc_t*)process_line);
@@ -3032,8 +3050,8 @@ int main(int argc, char* argv[], char* envp[])
 {
 	std::string cmd = argv[0];
 	std::cout << PACKAGE_STRING <<
-		", (c) 2002-2004 Heiko Stamer <stamer@gaos.org>, GNU GPL" << std::endl <<
-		" $Id: SecureSkat.cc,v 1.9 2005/01/06 20:00:21 stamer Exp $ " << std::endl;
+		", (c) 2002, 2005  Heiko Stamer <stamer@gaos.org>, GNU GPL" << std::endl <<
+		" $Id: SecureSkat.cc,v 1.15 2005/04/12 20:44:22 stamer Exp $ " << std::endl;
 	
 #ifdef ENABLE_NLS
 #ifdef HAVE_LC_MESSAGES
@@ -3079,6 +3097,14 @@ int main(int argc, char* argv[], char* envp[])
 			game_ctl = "";
 			game_env = NULL;
 		}
+		
+		// initalize libTMCG
+		if (!init_libTMCG())
+		{
+			std::cerr << _("Initalization of libTMCG failed!") << std::endl;
+			return -1;
+		}
+		
 		tmcg = new SchindelhauerTMCG(security_level, 3, 5); // 3 players, 32 cards
 		get_public_keys(cmd + ".pkr", nick_key);
 		get_secret_key(cmd + ".skr", sec, public_prefix);
@@ -3104,11 +3130,12 @@ int main(int argc, char* argv[], char* envp[])
 		release_pki(pki7771_handle);
 		set_public_keys(cmd + ".pkr", nick_key);
 		delete tmcg;
+		
 		return 0;
 	}
 	
-	std::cout << _("Usage: ") << cmd << " IRC_SERVER<std::string> [ IRC_PORT<int> " <<
+	std::cout << _("Usage: ") << cmd << " IRC_SERVER<string> [ IRC_PORT<int> " <<
 		"[ SECURITY_LEVEL<int> ..." << std::endl;
-	std::cout << "       " << " ... [ CONTROL_PROGRAM<std::string> ] ] ]" << std::endl;
+	std::cout << "       " << " ... [ CONTROL_PROGRAM<string> ] ] ]" << std::endl;
 	return -1;
 }
